import logging
import logging.handlers
from dataEngine import AppConfig

NOTICE = 25
logger = logging.getLogger("pbx-data-engine")


def init_logger():
    handlers = []

    logging.addLevelName(NOTICE, "NOTICE")

    def notice(self, message, *args, **kws):
        if self.isEnabledFor(NOTICE):
            self._log(NOTICE, message, args, **kws)
    logging.Logger.notice = notice

    logger.setLevel("NOTICE")

    # Use only one handler due to system store both of them to the syslog in case if syslog handler enabled
    # So, we turn on console handler log only for start script manualy with debug options
    if AppConfig.isDaemon():
        s_handler = logging.handlers.SysLogHandler(address='/dev/log')
        s_format = logging.Formatter('%(name)s: [%(levelname)s] [%(filename)s:%(lineno)d] %(message)s')
        s_handler.setFormatter(s_format)
        handlers.append(s_handler)
    else:
        console_handler = logging.StreamHandler()
        console_handler.setFormatter(CustomFormatter())
        handlers.append(console_handler)

    if AppConfig.get("log_file"):
        f_handler = logging.FileHandler(AppConfig.get("log_file"))
        f_format = logging.Formatter('%(name)s: [%(levelname)s] [%(filename)s:%(lineno)d] %(message)s')
        f_handler.setFormatter(f_format)
        handlers.append(f_handler)

    logging.basicConfig(level=logging.DEBUG if AppConfig.get("full_debug") else logging.WARNING, handlers=handlers)

    if AppConfig.get("loglevel"):
        logger.setLevel(AppConfig.get("loglevel"))


def get():
    return logger


class CustomFormatter(logging.Formatter):
    light_grey = "\033[92m"
    grey = "\033[38;1m"
    green = "\033[32;21m"
    yellow = "\033[33;21m"
    red = "\033[31;21m"
    bold_red = "\033[31;1m"
    reset = "\033[0m"
    format=u'[%(asctime)s] [%(levelname)s] [%(filename)s:%(lineno)d] %(message)s'

    FORMATS = {
        logging.DEBUG: green + format + reset,
        logging.INFO: yellow + format + reset,
        logging.WARNING: bold_red + format + reset,
        logging.ERROR: red + format + reset,
        logging.CRITICAL: bold_red + format + reset,
        NOTICE: light_grey + format + reset
    }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)