import os
import socket
from threading import Thread
from dataEngine import Aws, AppConfig, Logger

logger = Logger.get()


def listen_socket(address):
    try:
        os.unlink(address)
    except OSError:
        if os.path.exists(address):
            raise

    with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as sock:
        logger.info("Starting up on {}".format(address))
        sock.bind(address)
        sock.listen(1)
        sock.settimeout(1.0)

        while True:
            if SOCKET.exit:
                break
            try:
                connection, client_address = sock.accept()
                with connection:
                    data = connection.recv(128)
                    if data:
                        data_str = data.decode("utf-8").replace("\n", "")
                        if data_str == "token":
                            connection.sendall(str.encode(Aws.get_token()))
                        elif data_str == "cache_ldap:off":
                            AppConfig.set("cache_ldap", False)
                            connection.sendall(str.encode("OK"))
                        elif data_str == "cache_ldap:on":
                            AppConfig.set("cache_ldap", True)
                            connection.sendall(str.encode("OK"))
                        elif data_str == "cache_ldap:get":
                            connection.sendall(str.encode(str(AppConfig.get("cache_ldap"))))
                        elif data_str.startswith("loglevel:"):
                            level = data_str.replace("loglevel:", "")
                            if level and level.upper() in ("DEBUG", "INFO", "WARNING", "ERROR", "NOTICE"):
                                logger.setLevel(level.upper())
                                connection.sendall(str.encode("Logger set to {}".format(level)))
                            else:
                                connection.sendall(str.encode("Level {} is wrong".format(level)))
                        else:
                            logger.warning("Command '{}' is not implemented yet".format(data_str))
                            connection.sendall(str.encode("ERROR: Command '{}' is not implemented yet".format(data_str)))
            except socket.timeout as e:
                pass


def init_connection():
    SOCKET.thread.start()


def close_connection():
    SOCKET.exit = True
    SOCKET.thread.join()


class Socket():
    def __init__(self):
        self.thread = Thread(target=listen_socket, args=('/var/run/data_engine/data_engine.sock',))
        self.exit = False


SOCKET = Socket()
